function h5dump2(fileName,obj)
% H5DUMP2:  prints hdf5 metadata
%
%     H5DUMP(FILENAME) prints the entire file's metadata.
%
%     H5DUMP(FILENAME,OBJ) prints the metadata for the specified
%     object.  If obj is a group, all objects below the group will
%     be described.

error(nargchk(1,2,nargin,'struct'));

global indent_char;

indent_char = char(9);
indent_char = '   ';

file = H5F.open (fileName, 'H5F_ACC_RDONLY', 'H5P_DEFAULT');

if nargin == 1
	obj = '/';
end

op_func(file,obj);

H5F.close (file);

return


%--------------------------------------------------------------------------
function status=op_func (loc_id, name)
% This is the master iteration function.  It just basically serves as a
% switchyard.  If the current object is a group, the iteration recurses
% there.

statbuf=H5G.get_objinfo (loc_id, name, 0);

switch (statbuf.type)
    case H5ML.get_constant_value('H5G_GROUP')
        dump_group(loc_id,name);
        
    case H5ML.get_constant_value('H5G_DATASET')
        dump_dataset(loc_id,name,1);
        
    case H5ML.get_constant_value('H5G_TYPE')
        dump_datatype(loc_id,name);
        
    case H5ML.get_constant_value('H5G_LINK')
        dump_link(loc_id,name,1);
        
    otherwise
        fprintf ( '  Unknown: %s\n', name);
end

status=0;

return

%--------------------------------------------------------------------------
function status=attr_iterator (loc_id, name)

fprintf ('  Attribute: ''%s''\n', name);
status=0;

return

%--------------------------------------------------------------------------
function dump_link(loc_id,name, indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);
indent_p1 = repmat(indent_char, 1,indent_level+1);

%statbuf=H5G.get_objinfo (loc_id, name, 0)
buf = H5G.get_linkval(loc_id,name);

fprintf(1,'%sSoft Link:  ''%s''==>''%s''\n',indent, name,buf);

%--------------------------------------------------------------------------
function dump_datatype(dtype,name, indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);

dclass = H5T.get_class(dtype);
sz = H5T.get_size(dtype);

dtypeStr = get_datatype_string(dtype,indent_level);

fprintf ('%sDatatype:  %s\n', indent, dtypeStr);

return

%--------------------------------------------------------------------------
function dtypeStr = get_datatype_string(dtype,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);
indent_p1 = repmat(indent_char, 1,indent_level+1);

% Check vlen strings first.
if H5T.is_variable_str(dtype)
    dtypeStr = get_vlen_string(dtype,indent_level);
	return
end

class_id = H5T.get_class(dtype);

switch ( class_id )
    case H5ML.get_constant_value('H5T_BITFIELD')
        if H5T.equal(dtype,'H5T_STD_B8BE')
            dtypeStr = 'H5T_STD_B8BE';
        elseif H5T.equal(dtype,'H5T_STD_B8LE')
            dtypeStr = 'H5T_STD_B8LE';
        elseif H5T.equal(dtype,'H5T_STD_B16LE')
            dtypeStr = 'H5T_STD_B16LE';
        elseif H5T.equal(dtype,'H5T_STD_B16BE')
            dtypeStr = 'H5T_STD_B16BE';
        elseif H5T.equal(dtype,'H5T_STD_B32LE')
            dtypeStr = 'H5T_STD_B32LE';
        elseif H5T.equal(dtype,'H5T_STD_B32BE')
            dtypeStr = 'H5T_STD_B32BE';
        elseif H5T.equal(dtype,'H5T_STD_B64LE')
            dtypeStr = 'H5T_STD_B64LE';
        elseif H5T.equal(dtype,'H5T_STD_B64BE')
            dtypeStr = 'H5T_STD_B64BE';
        else
            error('HDF5TOOLS:h5dump:unrecognizedBitFieldType', ...
                  'Encountered an unrecognized bitfield type.' );
        end
    case H5ML.get_constant_value('H5T_INTEGER')
        if H5T.equal(dtype,'H5T_STD_I8BE')
            dtypeStr = 'H5T_STD_I8BE';
        elseif H5T.equal(dtype,'H5T_STD_I8LE')
            dtypeStr = 'H5T_STD_I8LE';
        elseif H5T.equal(dtype,'H5T_STD_U8BE')
            dtypeStr = 'H5T_STD_U8BE';
        elseif H5T.equal(dtype,'H5T_STD_U8LE')
            dtypeStr = 'H5T_STD_U8LE';
        elseif H5T.equal(dtype,'H5T_STD_I16BE')
            dtypeStr = 'H5T_STD_I16BE';
        elseif H5T.equal(dtype,'H5T_STD_I16LE')
            dtypeStr = 'H5T_STD_I16LE';
        elseif H5T.equal(dtype,'H5T_STD_U16BE')
            dtypeStr = 'H5T_STD_U16BE';
        elseif H5T.equal(dtype,'H5T_STD_U16LE')
            dtypeStr = 'H5T_STD_U16LE';
        elseif H5T.equal(dtype,'H5T_STD_I32BE')
            dtypeStr = 'H5T_STD_I32BE';
        elseif H5T.equal(dtype,'H5T_STD_I32LE')
            dtypeStr = 'H5T_STD_I32LE';
        elseif H5T.equal(dtype,'H5T_STD_U32BE')
            dtypeStr = 'H5T_STD_U32BE';
        elseif H5T.equal(dtype,'H5T_STD_U32LE')
            dtypeStr = 'H5T_STD_U32LE';
        elseif H5T.equal(dtype,'H5T_STD_I64BE')
            dtypeStr = 'H5T_STD_I64BE';
        elseif H5T.equal(dtype,'H5T_STD_I64LE')
            dtypeStr = 'H5T_STD_I64LE';
        elseif H5T.equal(dtype,'H5T_STD_U64BE')
            dtypeStr = 'H5T_STD_U64BE';
        elseif H5T.equal(dtype,'H5T_STD_U64LE')
            dtypeStr = 'H5T_STD_U64LE';
        else
            error('HDF5TOOLS:h5dump:unrecognizedIntegerType', ...
                  'Encountered an unrecognized integer type.' );
        end

    case H5ML.get_constant_value('H5T_FLOAT')
        if H5T.equal(dtype,'H5T_IEEE_F32BE')
            dtypeStr = 'H5T_IEEE_F32BE';
        elseif H5T.equal(dtype,'H5T_IEEE_F32LE')
            dtypeStr = 'H5T_IEEE_F32LE';
        elseif H5T.equal(dtype,'H5T_IEEE_F64BE')
            dtypeStr = 'H5T_IEEE_F64BE';
        elseif H5T.equal(dtype,'H5T_IEEE_F64LE')
            dtypeStr = 'H5T_IEEE_F64LE';
        else
            error('HDF5TOOLS:h5dump:unrecognizedFloatType', ...
                  'Encountered an unrecognized floating point type.' );
        end

    case H5ML.get_constant_value('H5T_OPAQUE')
        dtypeStr = get_opaque_string(dtype,indent_level);

    case H5ML.get_constant_value('H5T_REFERENCE')
        dtypeStr = 'H5T_REFERENCE';

    case H5ML.get_constant_value('H5T_COMPOUND')
        dtypeStr = get_compound_string(dtype,indent_level);


    case H5ML.get_constant_value('H5T_ENUM')
        dtypeStr = get_enum_string(dtype,indent_level);


    case H5ML.get_constant_value('H5T_ARRAY')
        dtypeStr = get_array_string(dtype,indent_level);

    case H5ML.get_constant_value('H5T_VLEN')
        dtypeStr = dump_vlen_datatype(dtype,indent_level+1);

    case H5ML.get_constant_value('H5T_STRING')
        dtypeStr = 'H5T_STRING';

    otherwise
        error('HDF5TOOLS:h5dump:unrecognizedClass', ...
              'Encountered an unrecognized datatype class %d.', ...
              class_id);
end


return

%--------------------------------------------------------------------------
function dtypeStr = get_opaque_string(dtype,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);
indent_p1 = repmat(indent_char, 1,indent_level+1);

dtypeStr = 'H5T_OPAQUE';

tag = H5T.get_tag(dtype);
dtypeStr = sprintf('H5T_OPAQUE\n%sOpaque Tag: %s', indent, tag );

return


%--------------------------------------------------------------------------
function dtypeStr = get_compound_string(dtype,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);
indent_p1 = repmat(indent_char, 1,indent_level+1);

dtypeStr = 'H5T_COMPOUND';

n = H5T.get_nmembers(dtype);
if n>0
    dtypeStr = sprintf('%s {\n', dtypeStr);
    for j = 0:n-1
        membName = H5T.get_member_name(dtype,j);
        membType = H5T.get_member_type(dtype,j);
        membDtypeStr = get_datatype_string(membType,indent_level+1);
        dtypeStr = sprintf('%s%s%s "%s"\n',dtypeStr,indent_p1,membDtypeStr,membName);
        H5T.close(membType);
    end
    dtypeStr = sprintf('%s%s}', dtypeStr, indent);
end

%--------------------------------------------------------------------------
function dtypeStr = get_array_string(dtype,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);
indent_p1 = repmat(indent_char, 1,indent_level+1);

dtypeStr = 'H5T_ARRAY';

% What are the dimensions of the array?
[n,dims] = H5T.get_array_dims(dtype);
if (n>0)
    dimStr = num2str(dims(1));
    for j = 2:n
        dimStr = sprintf('%s x %d', dimStr, dims(j));
    end
    dtypeStr = sprintf('%s [%s]', dtypeStr,dimStr);
end

dtypeStr = sprintf('%s {\n', dtypeStr );

% What is the array datatype?
super_dtype = H5T.get_super(dtype);
super_dtype_str = get_datatype_string(super_dtype,indent_level+1);
H5T.close(super_dtype);
dtypeStr = sprintf('%s%s%s\n', dtypeStr, indent_p1, super_dtype_str);

dtypeStr = sprintf('%s%s}', dtypeStr, indent);

%--------------------------------------------------------------------------
function dtypeStr = get_enum_string(dtype,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);
indent_p1 = repmat(indent_char, 1,indent_level+1);

dtypeStr = sprintf('%s {\n', 'H5T_ENUM' );
super_dtype = H5T.get_super(dtype);
super_dtype_str = get_datatype_string(super_dtype,indent_level+1);
H5T.close(super_dtype);
dtypeStr = sprintf('%s%s%s;\n', dtypeStr, indent_p1, super_dtype_str);

n = H5T.get_nmembers(dtype);
for j = 0:n-1
    name = H5T.get_member_name(dtype,j);    
    value = H5T.get_member_value(dtype,j);    
    dtypeStr = sprintf('%s%s%-12s\t%4d\n', dtypeStr,indent_p1,name,j);
end

dtypeStr = sprintf('%s%s}', dtypeStr, indent);

return

%--------------------------------------------------------------------------
function dtypeStr = get_vlen_string(dtype,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);
indent_p1 = repmat(indent_char, 1,indent_level+1);

    dtypeStr = sprintf('%s {\n', 'H5T_STRING' );
    dtypeStr = sprintf('%s%sStrSize H5T_VARIABLE;\n',dtypeStr,indent_p1);

    strPad = H5T.get_strpad(dtype);
	switch ( strPad )
		case H5ML.get_constant_value('H5T_STR_NULLTERM')
    		dtypeStr = sprintf('%s%sStrPad H5T_STR_NULLTERM;\n', dtypeStr,indent_p1);

		case H5ML.get_constant_value('H5T_STR_NULLPAD')
    		dtypeStr = sprintf('%s%sStrPad H5T_STR_NULLPAD;\n', dtypeStr,indent_p1);

		case H5ML.get_constant_value('H5T_STR_SPACEPAD')
    		dtypeStr = sprintf('%s%sStrPad H5T_STR_SPACEPAD;\n', dtypeStr,indent_p1);

		otherwise
			error('HDF5TOOLS:h5dump:unrecognizedStrPad', ...
			      'Unrecognized character string storage mechanism %d.', ...
				  strPad );
	end

    cset = H5T.get_cset(dtype);
	switch ( cset )
		case H5ML.get_constant_value('H5T_CSET_ASCII')
		    dtypeStr = sprintf('%s%sCSet H5T_CSET_ASCII;\n', dtypeStr,indent_p1);
			
		otherwise
			error('HDF5TOOLS:h5dump:unrecognizedCset', ...
			      'Unrecognized character set type %d.', ...
				  cset );
	end

    ctypeStr = get_ctype_string(dtype);

    dtypeStr = sprintf('%s%s}', dtypeStr, indent);

return

%--------------------------------------------------------------------------
function ctypeStr = get_ctype_string(dtype)
    strPad = H5T.get_strpad(dtype);
    cset = H5T.get_cset(dtype);

    str_type = H5T.copy('H5T_C_S1');
    %H5T.set_size(str_type,-1); % variable
    H5T.set_size(str_type, H5ML.get_constant_value('H5T_VARIABLE')); % variable
    H5T.set_cset(str_type,cset);
    H5T.set_strpad(str_type,strPad);
    if H5T.equal(dtype,str_type)
        ctypeStr = 'H5T_C_S1';
        return
    end

    % change the endianness, see if they're equal
    order = H5T.get_order(dtype);
    H5T.set_order(str_type,order);
    if H5T.equal(dtype,str_type)
        ctypeStr = 'H5T_C_S1';
        return
    end

    % allright, gods dammit, is it the fortran type?
    H5T.close(str_type);

    str_type = H5T.copy('H5T_FORTRAN_S1');

    H5T.set_size(str_type, H5T.get_size(dtype) ); 
    H5T.set_cset(str_type,cset);
    H5T.set_strpad(str_type,strPad);
    if H5T.equal(dtype,str_type)
        ctypeStr = 'H5T_FORTRAN_S1';
        return
    end

    % change the endianness, see if they're equal
    order = H5T.get_order(dtype);
    H5T.set_order(str_type,order);
    if H5T.equal(dtype,str_type)
        ctypeStr = 'H5T_FORTRAN_S1';
        return
    end

    % what the frak
    error('HDF5TOOLS:h5dump:unknownCharType', ...
          'This string datatype is not recognized.' );

return

%--------------------------------------------------------------------------
function str = dump_vlen_datatype(dtype_id,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);

str = 'H5T_VLEN';

super_dtype = H5T.get_super(dtype_id);
super_str = get_datatype_string(super_dtype, indent_level+1);
H5T.close(super_dtype);

str = sprintf ( 'H5T_VLEN { %s }', super_str );
return

%--------------------------------------------------------------------------
function dump_attribute(attr_id,indent_level)
global indent_char;
indent = repmat(indent_char, 1,indent_level);
fprintf('%sAttribute: ''%s'' {\n', indent, H5A.get_name(attr_id) );

dtype = H5A.get_type(attr_id);
dump_datatype(dtype,'',indent_level+1);
H5T.close(dtype);

space_id = H5A.get_space(attr_id);
dump_dataspace(space_id,indent_level+1);
H5S.close(space_id);

fprintf('%s}\n',indent);
return

%--------------------------------------------------------------------------
function dump_attributes(loc_id,indent_level)
% iterate over the group attributes
%H5A.iterate(loc_id,[],@attr_iterator);
total_attrs = H5A.get_num_attrs(loc_id);
for j = 0:total_attrs-1
    attr_id = H5A.open_idx(loc_id,j);
    dump_attribute(attr_id,indent_level);
    H5A.close(attr_id);
end
return

%--------------------------------------------------------------------------
function dump_group(loc_id,name)

if (name ~= '/')
    fullname = sprintf('%s/%s', H5I.get_name(loc_id), name);
else
	fullname = name;
end

if (numel(fullname)>1) && strcmp(fullname(1:2),'//')
    fullname = fullname(2:end);
end

fprintf ('Group ''%s'' {\n', fullname);

gid = H5G.open(loc_id,name);
dump_attributes(gid,1);
H5G.iterate (loc_id, name,[] , @op_func);
H5G.close(gid);
fprintf ('}\n');
return

%--------------------------------------------------------------------------
function dump_dataspace(space_id,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);

[ndims,dims,maxdims] = H5S.get_simple_extent_dims(space_id);
if ndims == 0
    fprintf ('%sDataspace scalar\n', indent);
else
    str = sprintf('%.0f ', flipud(dims(:)));
    str = deblank(str);

    unlimited = H5ML.get_constant_value('H5S_UNLIMITED');
    strmax = '';
    for j = ndims:-1:1
        if maxdims(j) == 18446744073709551616 % what the hey??
            strmax = [strmax 'H5S_UNLIMITED '];
        else
            strmax = [strmax num2str(maxdims(j)) ' '];
        end
    end
    strmax = deblank(strmax);

    fprintf ('%sDataspace simple {[%s] / [%s]}\n', indent, str, strmax);
end

return

%--------------------------------------------------------------------------
function dump_layout(dset_id,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);

dcpl = H5D.get_create_plist(dset_id);

layout = H5P.get_layout(dcpl);
switch ( layout )
    case H5ML.get_constant_value('H5D_COMPACT')
		layout_string = sprintf('Layout:  Compact');

    case H5ML.get_constant_value('H5D_CONTIGUOUS')
		layout_string = sprintf('Layout:  Contiguous');

    case H5ML.get_constant_value('H5D_CHUNKED')
		layout_string = sprintf('Layout:  Chunked');
		[rank,dims] = H5P.get_chunk(dcpl);
		str = '';
    	for j = rank:-1:1
	        str = [str num2str(dims(j)) ' '];
	    end
		str = sprintf('{[%s]}', deblank(str));
		layout_string = sprintf ( '%s %s', layout_string, str);

end

fprintf ('%s%s\n', indent, layout_string);

return

%--------------------------------------------------------------------------
function dump_dataset(loc_id,name,indent_level)

global indent_char;
indent = repmat(indent_char, 1,indent_level);

fprintf ('%sDataset ''%s'' {\n', indent, name);

dset_id = H5D.open(loc_id,name);

dtype = H5D.get_type(dset_id);
dump_datatype(dtype,'',indent_level+1);
H5T.close(dtype);

space_id = H5D.get_space(dset_id);
dump_dataspace(space_id,indent_level+1);
H5S.close(space_id);

dump_layout(dset_id,indent_level+1);

dump_attributes(dset_id,2);


H5D.close(dset_id);

fprintf ('%s}\n', indent);
return
