function th5varput()

test_writeFullDouble();
test_writeDoubleWithFillValue();
%test_extend1d();
%test_extend2d();

test_regressionFillValue();

fprintf ( 1, 'All H5VARPUT tests succeeded.\n\n' );
delete('foo.h5');

%--------------------------------------------------------------------------
function test_regressionFillValue()
% Release 0.30 incorrectly handled fill values.  We need to check that the
% dataset creation fill value flag was set to H5D_FILL_VALUE_USER_DEFINED
% before the fill value checking is implemented.

create_testfile('foo.h5');

data = rand(8,10);
data(1) = 99;
h5varput('foo.h5','/nofill',data);
outData = h5varget('foo.h5','/nofill');

if isnan(outData(1))
	error ( 'failed to verify writing double precision dataset without fill value');
end

fprintf ( 1, 'Regression test:  Writing a double precision variable without fill value succeeded.\n' );

%--------------------------------------------------------------------------
function test_writeDoubleWithFillValue()

create_testfile('foo.h5');

data = rand(8,10);
data(1) = 99;
h5varput('foo.h5','/DS1',data);
outData = h5varget('foo.h5','/DSF');

if ~isnan(outData(1))
	error ( 'failed to verify writing double precision dataset with fill value');
end

fprintf ( 1, 'Writing a double precision variable with fill value succeeded.\n' );

%--------------------------------------------------------------------------
function test_writeFullDouble()
create_testfile('foo.h5');

expVal = rand(8,10);
h5varput('foo.h5','/DS1',expVal);
actVal = h5varget('foo.h5','/DS1');
if ~isequal(actVal,expVal) && isa(actData,'double') 
	error ( 'failed to verify double precision retrieval');
end

fprintf ( 1, 'Writing a double precision variable succeeded.\n' );

%--------------------------------------------------------------------------
function test_extend1d()
create_testfile('foo.h5');

% Ok, the initial size should be 10.  Let's write that, then extend, then
% write some more.
expVal = [0:9];
h5varput('foo.h5','/DSC1',expVal);
actVal = h5varget('foo.h5','/DSC1');
if ~isequal(actVal(:),expVal(:)) && isa(actVal,'double') 
	error ( 'failed to verify double precision 1D retrieval');
end

% Now we write something that will extend the dataset.
expVal = [5:14];
h5varput('foo.h5','/DSC1',5,10,expVal);

fprintf ( 1, 'Writing and extending a double precision 1D succeeded.\n' );

%--------------------------------------------------------------------------
function test_extend2d()
create_testfile('foo.h5');

% Ok, the initial size should be 10.  Let's write that, then extend, then
% write some more.
expVal = ones(10,20);
h5varput('foo.h5','/DSC2',expVal);
actVal = h5varget('foo.h5','/DSC2');
if ~isequal(actVal(:),expVal(:)) && isa(actVal,'double') 
	error ( 'failed to verify double precision 1D retrieval');
end

% Now we write something that will extend the dataset, but only in
% the 2nd dimension.
expVal = 2*ones(10,20);
h5varput('foo.h5','/DSC2',[5 0],[10 20],expVal);
actVal = h5varget('foo.h5','/DSC2',[5 0], [10 20]);

fprintf ( 1, 'Writing and extending a double precision 2D succeeded.\n' );

%--------------------------------------------------------------------------
function create_testfile(hfile)

create_plist = H5P.create('H5P_FILE_CREATE');
file_id = H5F.create(hfile, 'H5F_ACC_TRUNC',create_plist , 'H5P_DEFAULT');

% Create a 8x10 double precision dataset.  No fill value
dims = [8 10];
space_id = H5S.create_simple(2,fliplr(dims),fliplr(dims));

dtype = H5T.copy('H5T_NATIVE_DOUBLE');
dcpl = H5P.create('H5P_DATASET_CREATE');
dataset_id = H5D.create(file_id,'/nofill','H5T_NATIVE_DOUBLE', space_id,dcpl);
H5P.close(dcpl);
H5D.close(dataset_id);
H5T.close(dtype);
H5S.close(space_id);



% Create a 8x10 double precision dataset.  It's fill value is 99.
dims = [8 10];
space_id = H5S.create_simple(2,fliplr(dims),fliplr(dims));

dataset_id = H5D.create(file_id,'/DS1','H5T_NATIVE_DOUBLE', space_id, 'H5P_DEFAULT');
H5D.close(dataset_id);

dtype = H5T.copy('H5T_NATIVE_DOUBLE');
dcpl = H5P.create('H5P_DATASET_CREATE');
H5P.set_fill_value(dcpl,dtype,99);
dataset_id = H5D.create(file_id,'/DSF','H5T_NATIVE_DOUBLE', space_id,dcpl);
H5P.close(dcpl);
H5D.close(dataset_id);
H5T.close(dtype);
H5S.close(space_id);


% Create a 1D dataset with chunking enabled.
dcpl = H5P.create('H5P_DATASET_CREATE');
H5P.set_chunk(dcpl,5);

space_id = H5S.create_simple(1,10,{'H5S_UNLIMITED'});
dtype = H5T.copy('H5T_NATIVE_DOUBLE');
dataset_id = H5D.create(file_id,'/DSC1','H5T_NATIVE_DOUBLE', space_id,dcpl);
H5P.close(dcpl);
H5D.close(dataset_id);
H5T.close(dtype);
H5S.close(space_id);

% Create a 2D dataset with chunking enabled.
dcpl = H5P.create('H5P_DATASET_CREATE');
H5P.set_chunk(dcpl,[5 5]);

space_id = H5S.create_simple(2,[10 20],{'H5S_UNLIMITED','H5S_UNLIMITED'});
dtype = H5T.copy('H5T_NATIVE_DOUBLE');
dataset_id = H5D.create(file_id,'/DSC2','H5T_NATIVE_DOUBLE', space_id,dcpl);
H5P.close(dcpl);
H5D.close(dataset_id);
H5T.close(dtype);
H5S.close(space_id);


H5F.close(file_id);
H5P.close(create_plist);

